@@ -1,8 +1,8 @@
 /*
    +----------------------------------------------------------------------+
-   | PHP Version 5                                                        |
+   | PHP Version 7                                                        |
    +----------------------------------------------------------------------+
-   | Copyright (c) 1997-2013 The PHP Group                                |
+   | Copyright (c) 1997-2018 The PHP Group                                |
    +----------------------------------------------------------------------+
    | This source file is subject to version 3.01 of the PHP license,      |
    | that is bundled with this package in the file LICENSE, and is        |
@@ -32,6 +32,7 @@
 #define spl_ce_Aggregate     zend_ce_aggregate
 #define spl_ce_ArrayAccess   zend_ce_arrayaccess
 #define spl_ce_Serializable  zend_ce_serializable
+#define spl_ce_Countable     zend_ce_countable
 
 extern PHPAPI zend_class_entry *spl_ce_RecursiveIterator;
 extern PHPAPI zend_class_entry *spl_ce_RecursiveIteratorIterator;
@@ -51,7 +52,8 @@ extern PHPAPI zend_class_entry *spl_ce_EmptyIterator;
 extern PHPAPI zend_class_entry *spl_ce_AppendIterator;
 extern PHPAPI zend_class_entry *spl_ce_RegexIterator;
 extern PHPAPI zend_class_entry *spl_ce_RecursiveRegexIterator;
-extern PHPAPI zend_class_entry *spl_ce_Countable;
+extern PHPAPI zend_class_entry *spl_ce_CallbackFilterIterator;
+extern PHPAPI zend_class_entry *spl_ce_RecursiveCallbackFilterIterator;
 
 PHP_MINIT_FUNCTION(spl_iterators);
 
@@ -75,6 +77,8 @@ typedef enum {
 	DIT_RegexIterator,
 	DIT_RecursiveRegexIterator,
 #endif
+	DIT_CallbackFilterIterator,
+	DIT_RecursiveCallbackFilterIterator,
 	DIT_Unknown = ~0
 } dual_it_type;
 
@@ -114,54 +118,64 @@ typedef enum {
 	REGIT_MODE_MAX
 } regex_mode;
 
+typedef struct _spl_cbfilter_it_intern {
+	zend_fcall_info       fci;
+	zend_fcall_info_cache fcc;
+	zend_object           *object;
+} _spl_cbfilter_it_intern;
+
 typedef struct _spl_dual_it_object {
-	zend_object              std;
 	struct {
-		zval                 *zobject;
+		zval                 zobject;
 		zend_class_entry     *ce;
 		zend_object          *object;
 		zend_object_iterator *iterator;
 	} inner;
 	struct {
-		zval                 *data;
-		char                 *str_key;
-		uint                 str_key_len;
-		ulong                int_key;
-		int                  key_type; /* HASH_KEY_IS_STRING or HASH_KEY_IS_LONG */
-		int                  pos;
+		zval                 data;
+		zval                 key;
+		zend_long            pos;
 	} current;
 	dual_it_type             dit_type;
 	union {
 		struct {
-			long             offset;
-			long             count;
+			zend_long             offset;
+			zend_long             count;
 		} limit;
 		struct {
-			long             flags; /* CIT_* */
-			zval             *zstr;
-			zval             *zchildren;
-			zval             *zcache;
+			zend_long             flags; /* CIT_* */
+			zval             zstr;
+			zval             zchildren;
+			zval             zcache;
 		} caching;
 		struct {
-			zval                 *zarrayit;
+			zval                  zarrayit;
 			zend_object_iterator *iterator;
 		} append;
 #if HAVE_PCRE || HAVE_BUNDLED_PCRE
 		struct {
-			int              use_flags;
-			long             flags;
-			regex_mode       mode;
-			long             preg_flags;
+			zend_long        flags;
+			zend_long        preg_flags;
 			pcre_cache_entry *pce;
-			char             *regex;
+			zend_string      *regex;
+			regex_mode       mode;
+			int              use_flags;
 		} regex;
 #endif
+		_spl_cbfilter_it_intern *cbfilter;
 	} u;
+	zend_object              std;
 } spl_dual_it_object;
 
-typedef int (*spl_iterator_apply_func_t)(zend_object_iterator *iter, void *puser TSRMLS_DC);
+static inline spl_dual_it_object *spl_dual_it_from_obj(zend_object *obj) /* {{{ */ {
+	return (spl_dual_it_object*)((char*)(obj) - XtOffsetOf(spl_dual_it_object, std));
+} /* }}} */
 
-PHPAPI int spl_iterator_apply(zval *obj, spl_iterator_apply_func_t apply_func, void *puser TSRMLS_DC);
+#define Z_SPLDUAL_IT_P(zv)  spl_dual_it_from_obj(Z_OBJ_P((zv)))
+
+typedef int (*spl_iterator_apply_func_t)(zend_object_iterator *iter, void *puser);
+
+PHPAPI int spl_iterator_apply(zval *obj, spl_iterator_apply_func_t apply_func, void *puser);
 
 #endif /* SPL_ITERATORS_H */
 
